/*
 *
 *  MULti Core Operating System - MultiLayer Kernel (MULCOS-M)
 *                   System structure definitions
 *                   Dependent on CortexM architecture
 *
 *               Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       arch_config.h
 *
 *  May/15/2024 Created.
 *
 */


#ifndef ARCH_CONFIG_H
#define ARCH_CONFIG_H


#include "typedef.h"

#if (__ARM_ARCH == 8)

#include <arm_cmse.h>

#if defined(__BUILD_SECURE__)
#if defined(__GNUC__)
#define GET_SECURE_BIT(para)  asm volatile ("ands %0, lr, #0x1;" : "=r"(para) : : )
#define CMSE_NS_CALL  __attribute((cmse_nonsecure_call))
#define CMSE_NS_ENTRY __attribute((cmse_nonsecure_entry))
typedef void CMSE_NS_CALL (*FUNC_NS_START)(void);
typedef void CMSE_NS_CALL (*FUNC_NS_RCALL)(uint32_t);
typedef int32_t CMSE_NS_CALL (*FUNC_NS_CALL)(uint32_t);
#elif defined ( __ICCARM__ )
#define GET_SECURE_BIT(para)  asm volatile ("ands %0, lr, #0x1;" : "=r"(para) : : )
#define CMSE_NS_CALL  __cmse_nonsecure_call
#define CMSE_NS_ENTRY __cmse_nonsecure_entry
typedef void (CMSE_NS_CALL *FUNC_NS_START)(void);
typedef void (CMSE_NS_CALL *FUNC_NS_RCALL)(uint32_t);
typedef int32_t (CMSE_NS_CALL *FUNC_NS_CALL)(uint32_t);
#else
#error "Can not support !!"
#endif
#define SECURE_FUNC   CMSE_NS_ENTRY
#else
#define SECURE_FUNC
#endif
#endif

typedef struct t_config_arch {
    uint32_t                fpscr;              // fpscr initial value
} T_CONFIG_ARCH;

#endif
