/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for STM32H56x GTZC
 *
 *           Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_stm32h56x_gtzc.h
 *
 *  Jan/12/2024     Created.
 *
 */

#ifndef _DV_STM32H56X_GTZC_H_
#define _DV_STM32H56X_GTZC_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define GTZC1_BASE            0x40032400            /* GTZC base address              */
#define GTZC1_TZSC_BASE       (GTZC1_BASE+0x0000)   /* TrustZone security controller  */
#define GTZC1_TZIC_BASE       (GTZC1_BASE+0x0400)   /* TrustZone illegal access controller */
#define GTZC1_MPCBB1_BASE     (GTZC1_BASE+0x0800)   /* Memory protection controller 1 */
#define GTZC1_MPCBB2_BASE     (GTZC1_BASE+0x0C00)   /* Memory protection controller 2 */
#define GTZC1_MPCBB3_BASE     (GTZC1_BASE+0x1000)   /* Memory protection controller 3 */

#define S_BASE_OFFSET   0x10000000      /* Secure peripheral offset address */

/* Registers */

typedef struct st_stm32h56x_gtzc_tzsc_t {
    UW   CR;            /* GTZC TZSC control register                             */
    UW   reserved0[3];  /* Reserved                                               */
    UW   SECCFGR1;      /* GTZC TZSC secure configuration register 1              */
    UW   SECCFGR2;      /* GTZC TZSC secure configuration register 2              */
    UW   SECCFGR3;      /* GTZC TZSC secure configuration register 3              */
    UW   reserved1;     /* Reserved                                               */
    UW   PRIVCFGR1;     /* GTZC TZSC privilege configuration register 1           */
    UW   PRIVCFGR2;     /* GTZC TZSC privilege configuration register 2           */
    UW   PRIVCFGR3;     /* GTZC TZSC privilege configuration register 3           */
    UW   reserved2[5];  /* Reserved                                               */
    UW   MPCWM1ACFGR;   /* GTZC TZSC memory 1 subregion A watermark configuration */
    UW   MPCWM1AR;      /* GTZC TZSC memory 1 subregion A watermark register      */
    UW   MPCWM1BCFGR;   /* GTZC TZSC memory 1 subregion B watermark configuration */
    UW   MPCWM1BR;      /* GTZC TZSC memory 1 subregion B watermark register      */
    UW   MPCWM2ACFGR;   /* GTZC TZSC memory 2 subregion A watermark configuration */
    UW   MPCWM2AR;      /* GTZC TZSC memory 2 subregion A watermark register      */
    UW   MPCWM2BCFGR;   /* GTZC TZSC memory 2 subregion B watermark configuration */
    UW   MPCWM2BR;      /* GTZC TZSC memory 2 subregion B watermark register      */
    UW   MPCWM3ACFGR;   /* GTZC TZSC memory 3 subregion A watermark configuration */
    UW   MPCWM3AR;      /* GTZC TZSC memory 3 subregion A watermark register      */
    UW   MPCWM3BCFGR;   /* GTZC TZSC memory 3 subregion B watermark configuration */
    UW   MPCWM3BR;      /* GTZC TZSC memory 3 subregion B watermark register      */
    UW   MPCWM4ACFGR;   /* GTZC TZSC memory 4 subregion A watermark configuration */
    UW   MPCWM4AR;      /* GTZC TZSC memory 4 subregion A watermark register      */
    UW   MPCWM4BCFGR;   /* GTZC TZSC memory 4 subregion B watermark configuration */
    UW   MPCWM4BR;      /* GTZC TZSC memory 4 subregion B watermark register      */

} T_STM32H56X_GTZC_TZSC;

typedef struct st_stm32h56x_gtzc_tzic_t {
    UW  IER1;           /* GTZC TZIC interrupt enable register 1                  */
    UW  IER2;           /* GTZC TZIC interrupt enable register 2                  */
    UW  IER3;           /* GTZC TZIC interrupt enable register 3                  */
    UW  IER4;           /* GTZC TZIC interrupt enable register 4                  */
    UW  SR1;            /* GTZC TZIC status register 1                            */
    UW  SR2;            /* GTZC TZIC status register 2                            */
    UW  SR3;            /* GTZC TZIC status register 3                            */
    UW  SR4;            /* GTZC TZIC status register 4                            */
    UW  FCR1;           /* GTZC TZIC flag clear register 1                        */
    UW  FCR2;           /* GTZC TZIC flag clear register 2                        */
    UW  FCR3;           /* GTZC TZIC flag clear register 3                        */
    UW  FCR4;           /* GTZC TZIC flag clear register 4                        */

} T_STM32H56X_GTZC_TZIC;

typedef struct st_stm32h56x_gtzc_mpcbb_t {
    UW   CR;            /* GTZC SRAM n MPCBB control register                     */
    UW   reserved0[3];  /* Reserved                                               */
    UW   CFGLOCK1;      /* GTZC SRAM n MPCBB configuration lock register 1        */
    UW   reserved1[59]; /* Reserved                                               */
    UW   SECCFGR[32];   /* GTZC SRAM n MPCBB secure configuration                 */
                        /*                          for super-block register      */
    UW   reserved2[32]; /* Reserved                                               */
    UW   PRIVCFGR[32];  /* GTZC SRAM n MPCBB privileged configuration             */
                        /*                          for super-block register      */

} T_STM32H56X_GTZC_MPCBB;


/* Port access structure */

#if defined(__BUILD_SECURE__)
#define REG_GTZC_TZSC       (*(volatile T_STM32H56X_GTZC_TZSC  *)(GTZC1_TZSC_BASE|S_BASE_OFFSET))
#define REG_GTZC_TZIC       (*(volatile T_STM32H56X_GTZC_TZIC  *)(GTZC1_TZIC_BASE|S_BASE_OFFSET))
#define REG_GTZC_MPCBB1     (*(volatile T_STM32H56X_GTZC_MPCBB *)(GTZC1_MPCBB1_BASE|S_BASE_OFFSET))
#define REG_GTZC_MPCBB2     (*(volatile T_STM32H56X_GTZC_MPCBB *)(GTZC1_MPCBB2_BASE|S_BASE_OFFSET))
#define REG_GTZC_MPCBB3     (*(volatile T_STM32H56X_GTZC_MPCBB *)(GTZC1_MPCBB3_BASE|S_BASE_OFFSET))

#define REG_GTZC_TZSC_NS    (*(volatile T_STM32H56X_GTZC_TZSC  *)GTZC1_TZSC_BASE)
#define REG_GTZC_TZIC_NS    (*(volatile T_STM32H56X_GTZC_TZIC  *)GTZC1_TZIC_BASE)
#define REG_GTZC_MPCBB1_NS  (*(volatile T_STM32H56X_GTZC_MPCBB *)GTZC1_MPCBB1_BASE)
#define REG_GTZC_MPCBB2_NS  (*(volatile T_STM32H56X_GTZC_MPCBB *)GTZC1_MPCBB2_BASE)
#define REG_GTZC_MPCBB3_NS  (*(volatile T_STM32H56X_GTZC_MPCBB *)GTZC1_MPCBB3_BASE)
#else
#define REG_GTZC_TZSC       (*(volatile T_STM32H56X_GTZC_TZSC  *)GTZC1_TZSC_BASE)
#define REG_GTZC_TZIC       (*(volatile T_STM32H56X_GTZC_TZIC  *)GTZC1_TZIC_BASE)
#define REG_GTZC_MPCBB1     (*(volatile T_STM32H56X_GTZC_MPCBB *)GTZC1_MPCBB1_BASE)
#define REG_GTZC_MPCBB2     (*(volatile T_STM32H56X_GTZC_MPCBB *)GTZC1_MPCBB2_BASE)
#define REG_GTZC_MPCBB3     (*(volatile T_STM32H56X_GTZC_MPCBB *)GTZC1_MPCBB3_BASE)
#endif

#ifdef __cplusplus
}
#endif
#endif /* _DV_STM32H56X_GTZC_H_ */
