/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for STM32H5xx ICACHE & DCACHE
 *
 *           Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_stm32h5xx_cache.h
 *
 *  Jan/05/2024     Created.
 *
 */

#ifndef _DV_STM32H5XX_CACHE_H_
#define _DV_STM32H5XX_CACHE_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define ICACHE_BASE     0x40030400      /* ICACHE Base address    */
#define DCACHE_BASE     0x40031400      /* DCACHE Base address    */

#define S_BASE_OFFSET   0x10000000      /* Secure peripheral offset address */

/* Registers */

typedef struct
{
    UW  CR;             /* ICACHE control register                */
    UW  SR;             /* ICACHE ICACHE status register          */
    UW  IER;            /* ICACHE interrupt enable register       */
    UW  FCR;            /* ICACHE flag clear register             */
    UW  HMONR;          /* ICACHE hit monitor register            */
    UW  MMONR;          /* ICACHE miss monitor register           */

} T_STM32H5XX_ICACH;

typedef struct
{
    UW  CR;             /* DCACHE control register                */
    UW  SR;             /* DCACHE ICACHE status register          */
    UW  IER;            /* DCACHE interrupt enable register       */
    UW  FCR;            /* DCACHE flag clear register             */
    UW  RHMONR;         /* DCACHE read-hit monitor register       */
    UW  RMMONR;         /* DCACHE read-miss monitor register      */
    UW  reserved1[2];
    UW  WHMONR;         /* DCACHE write-hit monitor register      */
    UW  WMMONR;         /* DCACHE write-miss monitor register     */

} T_STM32H5XX_DCACH;


/* Port access structure */

#if (defined(__BUILD_SECURE__) && !defined(_DV_STM32H50X_H_))
#define REG_ICACHE      (*(volatile T_STM32H5XX_ICACH *)(ICACHE_BASE|S_BASE_OFFSET))
#define REG_DCACHE      (*(volatile T_STM32H5XX_DCACH *)(DCACHE_BASE|S_BASE_OFFSET))
#define REG_ICACHE_NS   (*(volatile T_STM32H5XX_ICACH *)ICACHE_BASE)
#define REG_DCACHE_NS   (*(volatile T_STM32H5XX_DCACH *)DCACHE_BASE)
#else
#define REG_ICACHE      (*(volatile T_STM32H5XX_ICACH *)ICACHE_BASE)
#if !defined(_DV_STM32H50X_H_)
#define REG_DCACHE      (*(volatile T_STM32H5XX_DCACH *)DCACHE_BASE)
#endif
#endif


#ifdef __cplusplus
}
#endif
#endif /* _DV_STM32H5XX_CACHE_H_ */
