/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for STM32H5xx EXTI
 *
 *           Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_stm32h5xx_exti.h
 *
 *  Jan/08/2024     Created.
 *
 */

#ifndef _DV_STM32H7XX_EXTI_H_
#define _DV_STM32H7XX_EXTI_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define EXTI_BASE       0x44022000      /* EXTI base address */

#define S_BASE_OFFSET   0x10000000      /* Secure peripheral offset address */

/* Registers */

typedef struct st_stm32h5xx_exti_t {
    UW  RSTR1;          /* EXTI rising trigger selection register 1           */
    UW  FSTR1;          /* EXTI falling trigger selection register 1          */
    UW  SWIER1;         /* EXTI software interrupt event register 1           */
    UW  RPR1;           /* EXTI rising edge pending register 1                */
    UW  FPR1;           /* EXTI falling edge pending register 1               */
    UW  reserved0;      /* Reserved                                           */
    UW  PRIVCFGR1;      /* EXTI privilege configuration register 1            */
    UW  reserved1;      /* Reserved                                           */
    UW  RSTR2;          /* EXTI rising trigger selection register 2           */
    UW  FSTR2;          /* EXTI falling trigger selection register 2          */
    UW  SWIER2;         /* EXTI software interrupt event register 2           */
    UW  RPR2;           /* EXTI rising edge pending register 2                */
    UW  FPR2;           /* EXTI falling edge pending register 2               */
    UW  reserved2;      /* Reserved                                           */
    UW  PRIVCFGR2;      /* EXTI privilege configuration register 2            */
    UW  reserved3[9];   /* Reserved                                           */
    UW  EXTICR1;        /* EXTI external interrupt selection register 1       */
    UW  EXTICR2;        /* EXTI external interrupt selection register 2       */
    UW  EXTICR3;        /* EXTI external interrupt selection register 3       */
    UW  EXTICR4;        /* EXTI external interrupt selection register 4       */
    UW  reserved4[4];   /* Reserved                                           */
    UW  IMR1;           /* EXTI CPU wakeup with interrupt mask register 1     */
    UW  EMR1;           /* EXTI CPU wakeup with event mask register 1         */
    UW  reserved5[2];   /* Reserved                                           */
    UW  IMR2;           /* EXTI CPU wakeup with interrupt mask register 2     */
    UW  EMR2;           /* EXTI CPU wakeup with event mask register 2         */
} T_STM32H5XX_EXTI;


/* Port access structure */

#if (defined(__BUILD_SECURE__) && !defined(_DV_STM32H50X_H_))
#define REG_EXTI        (*(volatile T_STM32H5XX_EXTI *)(EXTI_BASE|S_BASE_OFFSET))
#define REG_EXTI_NS     (*(volatile T_STM32H5XX_EXTI *)EXTI_BASE)
#else
#define REG_EXTI        (*(volatile T_STM32H5XX_EXTI *)EXTI_BASE)
#endif

#ifdef __cplusplus
}
#endif
#endif /* _DV_STM32H5XX_EXTI_H_ */
