/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for STM32H5xx FLASH
 *
 *           Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_stm32h5xx_flash.h
 *
 *  Jan/01/2024     Created.
 *
 */

#ifndef _DV_STM32H5XX_FLASH_H_
#define _DV_STM32H5XX_FLASH_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define FLASH_BASE      0x40022000      /*FLASH Base address */

#define S_BASE_OFFSET   0x10000000      /* Secure peripheral offset address */

/* Registers */

typedef struct
{
    UW  ACR;            /* Flash access control register                     */
    UW  NSKEYR;         /* Flash key register                                */
    UW  reserved0;      /* Reserved                                          */
    UW  OPTKEYR;        /* Flash option key register                         */
    UW  reserved1[2];   /* Reserved                                          */
    UW  OPSR;           /* Flash operation status register                   */
    UW  OPTCR;          /* Flash option control register                     */
    UW  NSSR;           /* Flash status register                             */
    UW  reserved2;      /* Reserved                                          */
    UW  NSCR;           /* Flash control register                            */
    UW  reserved3;      /* Reserved                                          */
    UW  NSCCR;          /* Flash clear control register                      */
    UW  reserved4[2];   /* Reserved                                          */
    UW  PRIVCFGR;       /* Flash privilege configuration register            */
    UW  reserved5[2];   /* Reserved                                          */
    UW  HDPEXTR;        /* Flash HDP extension register                      */
    UW  reserved6;      /* Reserved                                          */
    UW  OPTSR_CUR;      /* Flash option status register                      */
    UW  OPTSR_PRG;      /* Flash option status register                      */
    UW  reserved7[6];   /* Reserved                                          */
    UW  OPTSR2_CUR;     /* Flash option status register 2                    */
    UW  OPTSR2_PRG;     /* Flash option status register 2                    */
    UW  reserved8[2];   /* Reserved                                          */
    UW  NBOOTR_CUR;     /* Flash unique boot entry register                  */
    UW  NBOOTR_PRG;     /* Flash unique boot entry address                   */
    UW  reserved9[2];   /* Reserved                                          */
    UW  OPTBLR_CUR;     /* Flash OTP block lock                              */
    UW  OPTBLR_PRG;     /* Flash OTP block lock                              */
    UW  reserved10[10]; /* Reserved                                          */
    UW  PRIVBB1R1;      /* privilege register for bank 1                     */
    UW  reserved11[9];  /* Reserved                                          */
    UW  WRP1R_CUR;      /* write sector protection for Bank1                 */
    UW  WRP1R_PRG;      /* write sector protection for Bank1                 */
    UW  reserved12[2];  /* Reserved                                          */
    UW  HDP1R_CUR;      /* HDP Bank1 register                                */
    UW  HDP1R_PRG;      /* HDP Bank1 register                                */
    UW  ECCCORR;        /* ECC correction register                           */
    UW  ECCDETR;        /* ECC detection register                            */
    UW  ECCDR;          /* ECC data                                          */
    UW  reserved13[45]; /* Reserved                                          */
    UW  PRIVBB2R1;      /* privilege register for Bank2                      */
    UW  reserved14[9];  /* Reserved                                          */
    UW  WRP2R_CUR;      /* write sector protection for Bank2                 */
    UW  WRP2R_PRG;      /* write sector protection for Bank2                 */
    UW  reserved15[2];  /* Reserved                                          */
    UW  HDP2R_CUR;      /* HDP Bank2 register                                */
    UW  HDP2R_PRG;      /* HDP Bank2 register                                */

} T_STM32H5XX_FLASH;


/* Port access structure */

#if (defined(__BUILD_SECURE__) && !defined(_DV_STM32H50X_H_))
#define REG_FLASH       (*(volatile T_STM32H5XX_FLASH *)(FLASH_BASE|S_BASE_OFFSET))
#define REG_FLASH_NS    (*(volatile T_STM32H5XX_FLASH *)FLASH_BASE)
#else
#define REG_FLASH       (*(volatile T_STM32H5XX_FLASH *)FLASH_BASE)
#endif

#ifdef __cplusplus
}
#endif
#endif /* _DV_STM32H5XX_FLASH_H_ */
