/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for STM32H5xx GPIO
 *
 *           Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_stm32h5xx_gpio.h
 *
 *  Jan/01/2024     Created.
 *
 */

#ifndef _DV_STM32H5XX_GPIO_H_
#define _DV_STM32H5XX_GPIO_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define GPIOA_BASE      0x42020000      /* GPIOA base address */
#define GPIOB_BASE      0x42020400      /* GPIOB base address */
#define GPIOC_BASE      0x42020800      /* GPIOC base address */
#define GPIOD_BASE      0x42020C00      /* GPIOD base address */
#define GPIOH_BASE      0x42021C00      /* GPIOH base address */
#define GPIOE_BASE      0x42021000      /* GPIOE base address */
#define GPIOF_BASE      0x42021400      /* GPIOF base address */
#define GPIOG_BASE      0x42021800      /* GPIOG base address */
#define GPIOI_BASE      0x42022000      /* GPIOI base address */

#define S_BASE_OFFSET   0x10000000      /* Secure peripheral offset address */

/* Registers */

typedef struct st_stm32h5xx_gpio_t {
    UW   MODER;         /* GPIO port mode register,                           */
    UW   OTYPER;        /* GPIO port output type register                     */
    UW   OSPEEDR;       /* GPIO port output speed register                    */
    UW   PUPDR;         /* GPIO port pull-up/pull-down register               */
    UW   IDR;           /* GPIO port input data register                      */
    UW   ODR;           /* GPIO port output data register                     */
    UW   BSRR;          /* GPIO port bit set/reset register                   */
    UW   LCKR;          /* GPIO port configuration lock register              */
    UW   AFR[2];        /* GPIO alternate function registers                  */
    UW   BRR;           /* GPIO port bit reset register                       */
    UW   HSLVR;         /* GPIO high-speed low-voltage register               */
#if (!defined(_DV_STM32H50X_H_))
    UW   SECCFGR;       /* GPIO secure configuration register                 */
#endif
} T_STM32H5XX_GPIO;


/* Port access structure */

#if defined(_DV_STM32H50X_H_)
#define REG_GPIOA       (*(volatile T_STM32H5XX_GPIO *)GPIOA_BASE)
#define REG_GPIOB       (*(volatile T_STM32H5XX_GPIO *)GPIOB_BASE)
#define REG_GPIOC       (*(volatile T_STM32H5XX_GPIO *)GPIOC_BASE)
#define REG_GPIOD       (*(volatile T_STM32H5XX_GPIO *)GPIOD_BASE)
#define REG_GPIOH       (*(volatile T_STM32H5XX_GPIO *)GPIOH_BASE)
#else
#if defined(__BUILD_SECURE__)
#define REG_GPIOA       (*(volatile T_STM32H5XX_GPIO *)(GPIOA_BASE|S_BASE_OFFSET))
#define REG_GPIOB       (*(volatile T_STM32H5XX_GPIO *)(GPIOB_BASE|S_BASE_OFFSET))
#define REG_GPIOC       (*(volatile T_STM32H5XX_GPIO *)(GPIOC_BASE|S_BASE_OFFSET))
#define REG_GPIOD       (*(volatile T_STM32H5XX_GPIO *)(GPIOD_BASE|S_BASE_OFFSET))
#define REG_GPIOH       (*(volatile T_STM32H5XX_GPIO *)(GPIOH_BASE|S_BASE_OFFSET))
#define REG_GPIOE       (*(volatile T_STM32H5XX_GPIO *)(GPIOE_BASE|S_BASE_OFFSET))
#define REG_GPIOF       (*(volatile T_STM32H5XX_GPIO *)(GPIOF_BASE|S_BASE_OFFSET))
#define REG_GPIOG       (*(volatile T_STM32H5XX_GPIO *)(GPIOG_BASE|S_BASE_OFFSET))
#define REG_GPIOI       (*(volatile T_STM32H5XX_GPIO *)(GPIOI_BASE|S_BASE_OFFSET))

#define REG_GPIOA_NS    (*(volatile T_STM32H5XX_GPIO *)GPIOA_BASE)
#define REG_GPIOB_NS    (*(volatile T_STM32H5XX_GPIO *)GPIOB_BASE)
#define REG_GPIOC_NS    (*(volatile T_STM32H5XX_GPIO *)GPIOC_BASE)
#define REG_GPIOD_NS    (*(volatile T_STM32H5XX_GPIO *)GPIOD_BASE)
#define REG_GPIOH_NS    (*(volatile T_STM32H5XX_GPIO *)GPIOH_BASE)
#define REG_GPIOE_NS    (*(volatile T_STM32H5XX_GPIO *)GPIOE_BASE)
#define REG_GPIOF_NS    (*(volatile T_STM32H5XX_GPIO *)GPIOF_BASE)
#define REG_GPIOG_NS    (*(volatile T_STM32H5XX_GPIO *)GPIOG_BASE)
#define REG_GPIOI_NS    (*(volatile T_STM32H5XX_GPIO *)GPIOI_BASE)
#else

#define REG_GPIOA       (*(volatile T_STM32H5XX_GPIO *)GPIOA_BASE)
#define REG_GPIOB       (*(volatile T_STM32H5XX_GPIO *)GPIOB_BASE)
#define REG_GPIOC       (*(volatile T_STM32H5XX_GPIO *)GPIOC_BASE)
#define REG_GPIOD       (*(volatile T_STM32H5XX_GPIO *)GPIOD_BASE)
#define REG_GPIOH       (*(volatile T_STM32H5XX_GPIO *)GPIOH_BASE)
#define REG_GPIOE       (*(volatile T_STM32H5XX_GPIO *)GPIOE_BASE)
#define REG_GPIOF       (*(volatile T_STM32H5XX_GPIO *)GPIOF_BASE)
#define REG_GPIOG       (*(volatile T_STM32H5XX_GPIO *)GPIOG_BASE)
#define REG_GPIOI       (*(volatile T_STM32H5XX_GPIO *)GPIOI_BASE)
#endif
#endif


#ifdef __cplusplus
}
#endif
#endif /* _DV_STM32H5XX_GPIO_H_ */
