/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for STM32H5xx PWR
 *
 *           Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_stm32h5xx_pwr.h
 *
 *  Jan/01/2024     Created.
 *
 */

#ifndef _DV_STM32H5XX_PWR_H_
#define _DV_STM32H5XX_PWR_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define PWR_BASE        0x44020800      /* PWR base address */

#define S_BASE_OFFSET   0x10000000      /* Secure peripheral offset address */

/* Registers */

typedef struct st_stm32h5xx_pwr_t {
    UW   PMCR;          /* PWR power mode control register                    */
    UW   PMSR;          /* PWR status register                                */
    UW   reserved0[2];  /* Reserved                                           */
    UW   VOSCR;         /* PWR voltage scaling control register               */
    UW   VOSSR;         /* PWR voltage scaling status register                */
    UW   reserved1[2];  /* Reserved                                           */
    UW   BDCR;          /* PWR backup domain control register                 */
    UW   DBPCR;         /* PWR disable backup protection control register     */
    UW   BDSR;          /* PWR backup domain status register                  */
    UW   reserved2;     /* Reserved                                           */
    UW   SCCR;          /* PWR supply configuration control register          */
    UW   VMCR;          /* PWR voltage monitor control register               */
    UW   reserved3;     /* Reserved                                           */
    UW   VMSR;          /* PWR voltage monitor status register                */
    UW   WUSCR;         /* PWR wakeup status clear register                   */
    UW   WUSR;          /* PWR wakeup status register                         */
    UW   WUCR;          /* PWR wakeup configuration register                  */
    UW   reserved4;     /* Reserved                                           */
    UW   IORETR;        /* PWR I/O retention register                         */
    UW   reserved5[44]; /* Reserved                                           */
    UW   PRIVCFGR;      /* PWR privilege configuration register               */

} T_STM32H5XX_PWR;


/* Port access structure */

#if (defined(__BUILD_SECURE__) && !defined(_DV_STM32H50X_H_))
#define REG_PWR         (*(volatile T_STM32H5XX_PWR *)(PWR_BASE|S_BASE_OFFSET))
#define REG_PWR_NS      (*(volatile T_STM32H5XX_PWR *)PWR_BASE)
#else
#define REG_PWR         (*(volatile T_STM32H5XX_PWR *)PWR_BASE)
#endif

#ifdef __cplusplus
}
#endif
#endif /* _DV_STM32H5XX_PWR_H_ */
