/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for STM32H5xx RCC
 *
 *           Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_stm32h5xx_rcc.h
 *
 *  Jan/01/2024     Created.
 *
 */

#ifndef _DV_STM32H5XX_RCC_H_
#define _DV_STM32H5XX_RCC_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define RCC_BASE        0x44020C00      /* RCC base address */

#define S_BASE_OFFSET   0x10000000      /* Secure peripheral offset address */

/* Registers */

typedef struct st_stm32h5xx_rcc_t {
    UW   CR;            /* RCC clock control register                         */
    UW   reserved0[3];  /* Reserved                                           */
    UW   HSICFGR;       /* RCC HSI configuration register                     */
    UW   CRRCR;         /* RCC clock recovery RC register                     */
    UW   CSICFGR;       /* RCC CSI configuration register                     */
    UW   CFGR1;         /* RCC clock configuration register 1                 */
    UW   CFGR2;         /* RCC clock configuration register 2                 */
    UW   reserved1;     /* Reserved                                           */
    UW   PLL1CFGR;      /* RCC PLL1 clock source selection register           */
    UW   PLL2CFGR;      /* RCC PLL2 clock source selection register           */
    UW   reserved2;     /* Reserved                                           */
    UW   PLL1DIVR;      /* RCC PLL1 dividers register                         */
    UW   PLL1FRACR;     /* RCC PLL1 fractional divider register               */
    UW   PLL2DIVR;      /* RCC PLL2 dividers register                         */
    UW   PLL2FRACR;     /* RCC PLL2 fractional divider register               */
    UW   reserved3[3];  /* Reserved                                           */
    UW   CIER;          /* RCC clock source interruput enable register        */
    UW   CIFR;          /* RCC clock source interruput flag register          */
    UW   CICR;          /* RCC clock source interruput clear register         */
    UW   reserved4;     /* Reserved                                           */
    UW   AHB1RSTR;      /* RCC AHB1 peripheral reset register                 */
    UW   AHB2RSTR;      /* RCC AHB2 peripheral reset register                 */
    UW   reserved5[3];  /* Reserved                                           */
    UW   APB1LRSTR;     /* RCC APB1 peripheral reset low register             */
    UW   APB1HRSTR;     /* RCC APB1 peripheral reset high register            */
    UW   APB2RSTR;      /* RCC APB2 peripheral reset register                 */
    UW   APB3RSTR;      /* RCC APB3 peripheral reset register                 */
    UW   reserved6;     /* Reserved                                           */
    UW   AHB1ENR;       /* RCC AHB1 peripheral clock register                 */
    UW   AHB2ENR;       /* RCC AHB2 peripheral clock register                 */
    UW   reserved7[3];  /* Reserved                                           */
    UW   APB1LENR;      /* RCC APB1 peripheral clock enable low register      */
    UW   APB1HENR;      /* RCC APB1 peripheral clock enable high register     */
    UW   APB2ENR;       /* RCC APB2 peripheral clock enable register          */
    UW   APB3ENR;       /* RCC APB3 peripheral clock enable register          */
    UW   reserved8;     /* Reserved                                           */
    UW   AHB1LPENR;     /* RCC AHB1 sleep clock register                      */
    UW   AHB2LPENR;     /* RCC AHB2 sleep clock register                      */
    UW   reserved9[3];  /* Reserved                                           */
    UW   APB1LLPENR;    /* RCC APB1 sleep clock low register                  */
    UW   APB1HLPENR;    /* RCC APB1 sleep clock high register                 */
    UW   APB2LPENR;     /* RCC APB2 sleep clock register                      */
    UW   APB3LPENR;     /* RCC APB3 sleep clock register                      */
    UW   reserved10;    /* Reserved                                           */
    UW   CCIPR1;        /* RCC kernel clock configuration register 1          */
    UW   CCIPR2;        /* RCC kernel clock configuration register 2          */
    UW   CCIPR3;        /* RCC kernel clock configuration register 3          */
    UW   CCIPR4;        /* RCC kernel clock configuration register 4          */
    UW   CCIPR5;        /* RCC kernel clock configuration register 5          */
    UW   reserved11;    /* Reserved                                           */
    UW   BDCR;          /* RCC Backup domain control register                 */
    UW   RSR;           /* RCC reset status register                          */
    UW   reserved12[7]; /* Reserved                                           */
    UW   PRIVCFGR;      /* RCC privilege configuration register               */

} T_STM32H5XX_RCC;


/* Port access structure */

#if (defined(__BUILD_SECURE__) && !defined(_DV_STM32H50X_H_))
#define REG_RCC         (*(volatile T_STM32H5XX_RCC *)(RCC_BASE|S_BASE_OFFSET))
#define REG_RCC_NS      (*(volatile T_STM32H5XX_RCC *)RCC_BASE)
#else
#define REG_RCC         (*(volatile T_STM32H5XX_RCC *)RCC_BASE)
#endif

#ifdef __cplusplus
}
#endif
#endif /* _DV_STM32F5XX_RCC_H_ */
