/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for STM32H5xx TIM
 *
 *           Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_stm32h5xx_tim.h
 *
 *  Jan/01/2024     Created.
 *
 */

#ifndef _DV_STM32H5XX_TIM_H_
#define _DV_STM32H5XX_TIM_H_
#ifdef __cplusplus
extern "C" {
#endif

/* Base Address */

#define TIM1_BASE       0x40012C00      /* TIM1 base address */
#define TIM2_BASE       0x40000000      /* TIM2 base address */
#define TIM3_BASE       0x40000400      /* TIM3 base address */
#define TIM4_BASE       0x40000800      /* TIM4 base address */
#define TIM5_BASE       0x40000C00      /* TIM5 base address */
#define TIM6_BASE       0x40001000      /* TIM6 base address */
#define TIM7_BASE       0x40001400      /* TIM7 base address */
#define TIM8_BASE       0x40013400      /* TIM8 base address */

#define TIM12_BASE      0x40001800      /* TIM12 base address */
#define TIM13_BASE      0x40001C00      /* TIM13 base address */
#define TIM14_BASE      0x40002000      /* TIM14 base address */
#define TIM15_BASE      0x40014000      /* TIM15 base address */
#define TIM16_BASE      0x40014400      /* TIM16 base address */
#define TIM17_BASE      0x40014800      /* TIM17 base address */

#define S_BASE_OFFSET   0x10000000      /* Secure peripheral offset address */

/* Registers */

typedef struct st_stm32h5xx_tim_t {
    UW   CR1;             /* TIM power mode control register                    */
    UW   CR2;             /* TIM status register                                */
    UW   SMCR;            /* TIM rved                                           */
    UW   DIER;            /* TIM voltage scaling control register               */
    UW   SR;              /* TIM voltage scaling status register                */
    UW   EGR;             /* TIM rved                                           */
    UW   CCMR1;           /* TIM backup domain control register                 */
    UW   CCMR2;           /* TIM disable backup protection control register     */
    UW   CCER;            /* TIM backup domain status register                  */
    UW   CNT;             /* TIM rved                                           */
    UW   PSC;             /* TIM supply configuration control register          */
    UW   ARR;             /* TIM voltage monitor control register               */
    UW   RCR;             /* TIM rved                                           */
    UW   CCR1;            /* TIM voltage monitor status register                */
    UW   CCR2;            /* TIM wakeup status clear register                   */
    UW   CCR3;            /* TIM wakeup status register                         */
    UW   CCR4;            /* TIM wakeup configuration register                  */
    UW   BDTR;            /* TIM rved                                           */
    UW   CCR5;            /* TIM I/O retention register                         */
    UW   CCR6;            /* TIM I/O retention register                         */
    UW   CCMR3;           /* TIM I/O retention register                         */
    UW   DTR2;            /* TIM I/O retention register                         */
    UW   ECR;             /* TIM I/O retention register                         */
    UW   TISEL;           /* TIM I/O retention register                         */
    UW   AF1;             /* TIM I/O retention register                         */
    UW   AF2;             /* TIM I/O retention register                         */
    UW   reserved0[221];  /* Reserved                                           */
    UW   DCR;             /* TIM privilege configuration register               */
    UW   DMAR;            /* TIM privilege configuration register               */

} T_STM32H5XX_TIM;


/* Port access structure */

#if defined(_DV_STM32H50X_H_)
#define REG_TIM1        (*(volatile T_STM32H5XX_TIM *)TIM1_BASE)
#define REG_TIM2        (*(volatile T_STM32H5XX_TIM *)TIM2_BASE)
#define REG_TIM3        (*(volatile T_STM32H5XX_TIM *)TIM3_BASE)
#define REG_TIM4        (*(volatile T_STM32H5XX_TIM *)TIM4_BASE)
#define REG_TIM5        (*(volatile T_STM32H5XX_TIM *)TIM5_BASE)
#define REG_TIM6        (*(volatile T_STM32H5XX_TIM *)TIM6_BASE)
#define REG_TIM7        (*(volatile T_STM32H5XX_TIM *)TIM7_BASE)
#define REG_TIM8        (*(volatile T_STM32H5XX_TIM *)TIM8_BASE)

#else
#if defined(__BUILD_SECURE__)
#define REG_TIM1        (*(volatile T_STM32H5XX_TIM *)(TIM1_BASE|S_BASE_OFFSET))
#define REG_TIM2        (*(volatile T_STM32H5XX_TIM *)(TIM2_BASE|S_BASE_OFFSET))
#define REG_TIM3        (*(volatile T_STM32H5XX_TIM *)(TIM3_BASE|S_BASE_OFFSET))
#define REG_TIM4        (*(volatile T_STM32H5XX_TIM *)(TIM4_BASE|S_BASE_OFFSET))
#define REG_TIM5        (*(volatile T_STM32H5XX_TIM *)(TIM5_BASE|S_BASE_OFFSET))
#define REG_TIM6        (*(volatile T_STM32H5XX_TIM *)(TIM6_BASE|S_BASE_OFFSET))
#define REG_TIM7        (*(volatile T_STM32H5XX_TIM *)(TIM7_BASE|S_BASE_OFFSET))
#define REG_TIM8        (*(volatile T_STM32H5XX_TIM *)(TIM8_BASE|S_BASE_OFFSET))
#define REG_TIM12       (*(volatile T_STM32H5XX_TIM *)(TIM12_BASE|S_BASE_OFFSET))
#define REG_TIM13       (*(volatile T_STM32H5XX_TIM *)(TIM13_BASE|S_BASE_OFFSET))
#define REG_TIM14       (*(volatile T_STM32H5XX_TIM *)(TIM14_BASE|S_BASE_OFFSET))
#define REG_TIM15       (*(volatile T_STM32H5XX_TIM *)(TIM15_BASE|S_BASE_OFFSET))
#define REG_TIM16       (*(volatile T_STM32H5XX_TIM *)(TIM16_BASE|S_BASE_OFFSET))
#define REG_TIM17       (*(volatile T_STM32H5XX_TIM *)(TIM17_BASE|S_BASE_OFFSET))

#define REG_TIM1_NS     (*(volatile T_STM32H5XX_TIM *)TIM1_BASE)
#define REG_TIM2_NS     (*(volatile T_STM32H5XX_TIM *)TIM2_BASE)
#define REG_TIM3_NS     (*(volatile T_STM32H5XX_TIM *)TIM3_BASE)
#define REG_TIM4_NS     (*(volatile T_STM32H5XX_TIM *)TIM4_BASE)
#define REG_TIM5_NS     (*(volatile T_STM32H5XX_TIM *)TIM5_BASE)
#define REG_TIM6_NS     (*(volatile T_STM32H5XX_TIM *)TIM6_BASE)
#define REG_TIM7_NS     (*(volatile T_STM32H5XX_TIM *)TIM7_BASE)
#define REG_TIM8_NS     (*(volatile T_STM32H5XX_TIM *)TIM8_BASE)
#define REG_TIM12_NS    (*(volatile T_STM32H5XX_TIM *)TIM12_BASE)
#define REG_TIM13_NS    (*(volatile T_STM32H5XX_TIM *)TIM13_BASE)
#define REG_TIM14_NS    (*(volatile T_STM32H5XX_TIM *)TIM14_BASE)
#define REG_TIM15_NS    (*(volatile T_STM32H5XX_TIM *)TIM15_BASE)
#define REG_TIM16_NS    (*(volatile T_STM32H5XX_TIM *)TIM16_BASE)
#define REG_TIM17_NS    (*(volatile T_STM32H5XX_TIM *)TIM17_BASE)
#else

#define REG_TIM1        (*(volatile T_STM32H5XX_TIM *)TIM1_BASE)
#define REG_TIM2        (*(volatile T_STM32H5XX_TIM *)TIM2_BASE)
#define REG_TIM3        (*(volatile T_STM32H5XX_TIM *)TIM3_BASE)
#define REG_TIM4        (*(volatile T_STM32H5XX_TIM *)TIM4_BASE)
#define REG_TIM5        (*(volatile T_STM32H5XX_TIM *)TIM5_BASE)
#define REG_TIM6        (*(volatile T_STM32H5XX_TIM *)TIM6_BASE)
#define REG_TIM7        (*(volatile T_STM32H5XX_TIM *)TIM7_BASE)
#define REG_TIM8        (*(volatile T_STM32H5XX_TIM *)TIM8_BASE)
#define REG_TIM12       (*(volatile T_STM32H5XX_TIM *)TIM12_BASE)
#define REG_TIM13       (*(volatile T_STM32H5XX_TIM *)TIM13_BASE)
#define REG_TIM14       (*(volatile T_STM32H5XX_TIM *)TIM14_BASE)
#define REG_TIM15       (*(volatile T_STM32H5XX_TIM *)TIM15_BASE)
#define REG_TIM16       (*(volatile T_STM32H5XX_TIM *)TIM16_BASE)
#define REG_TIM17       (*(volatile T_STM32H5XX_TIM *)TIM17_BASE)
#endif
#endif

#ifdef __cplusplus
}
#endif
#endif /* _DV_STM32H5XX_TIM_H_ */
