/*
 *
 *           MULti Core Operating System - Tiny / Multilayer Kernel
 *                      Device header for STM32H5xx UART
 *
 *           Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name       dv_stm32h5xx_uart.h
 *
 *  Jan/01/2024     Created.
 *
 */

#ifndef _DV_STM32H5XX_UART_H_
#define _DV_STM32H5XX_UART_H_
#ifdef __cplusplus
extern "C" {
#endif


/* Excetion number */

#define IRQ_UART1           ( 58+16)
#define IRQ_UART2           ( 59+16)
#define IRQ_UART3           ( 60+16)
#define IRQ_UART4           ( 61+16)
#define IRQ_UART5           ( 62+16)
#define IRQ_UART6           ( 85+16)
#define IRQ_UART7           ( 98+16)
#define IRQ_UART8           ( 99+16)
#define IRQ_UART9           (100+16)
#define IRQ_UART10          ( 86+16)
#define IRQ_UART11          ( 87+16)
#define IRQ_UART12          (101+16)

/* Base Address */

#define UART1_BASE      0x40013800      /* USART1 via APB2 */
#define UART2_BASE      0x40004400      /* USART2 via APB1 */
#define UART3_BASE      0x40004800      /* USART3 via APB1 */
#define UART4_BASE      0x40004C00      /* USART3 via APB1 */
#define UART5_BASE      0x40005000      /* USART3 via APB1 */
#define UART6_BASE      0x40006400      /* USART3 via APB1 */
#define UART7_BASE      0x40007800      /* USART3 via APB1 */
#define UART8_BASE      0x40007C00      /* USART3 via APB1 */
#define UART9_BASE      0x40008000      /* USART3 via APB1 */
#define UART10_BASE     0x40006800      /* USART3 via APB1 */
#define UART11_BASE     0x40006C00      /* USART3 via APB1 */
#define UART12_BASE     0x40008400      /* USART3 via APB1 */

#define S_BASE_OFFSET   0x10000000      /* Secure peripheral offset address */


/* Registers */

typedef struct st_stm32h5xx_uart {
    UW  cr1;
    UW  cr2;
    UW  cr3;
    UW  brr;
    UW  gtpr;
    UW  rtor;
    UW  rqr;
    UW  isr;
    UW  icr;
    UW  rdr;
    UW  tdr;
    UW  presc;
} T_STM32H5XX_UART;

/* Port Address */

#if defined(_DV_STM32H50X_H_)
#define REG_UART1       (*(volatile T_STM32H5XX_UART *)UART1_BASE)
#define REG_UART2       (*(volatile T_STM32H5XX_UART *)UART2_BASE)
#define REG_UART3       (*(volatile T_STM32H5XX_UART *)UART3_BASE)
#else
#if defined(__BUILD_SECURE__)
#define REG_UART1       (*(volatile T_STM32H5XX_UART *)(UART1_BASE|S_BASE_OFFSET))
#define REG_UART2       (*(volatile T_STM32H5XX_UART *)(UART2_BASE|S_BASE_OFFSET))
#define REG_UART3       (*(volatile T_STM32H5XX_UART *)(UART3_BASE|S_BASE_OFFSET))
#define REG_UART4       (*(volatile T_STM32H5XX_UART *)(UART4_BASE|S_BASE_OFFSET))
#define REG_UART5       (*(volatile T_STM32H5XX_UART *)(UART5_BASE|S_BASE_OFFSET))
#define REG_UART6       (*(volatile T_STM32H5XX_UART *)(UART6_BASE|S_BASE_OFFSET))
#define REG_UART7       (*(volatile T_STM32H5XX_UART *)(UART7_BASE|S_BASE_OFFSET))
#define REG_UART8       (*(volatile T_STM32H5XX_UART *)(UART8_BASE|S_BASE_OFFSET))
#define REG_UART9       (*(volatile T_STM32H5XX_UART *)(UART9_BASE|S_BASE_OFFSET))
#define REG_UART10      (*(volatile T_STM32H5XX_UART *)(UART10_BASE|S_BASE_OFFSET))
#define REG_UART11      (*(volatile T_STM32H5XX_UART *)(UART11_BASE|S_BASE_OFFSET))
#define REG_UART12      (*(volatile T_STM32H5XX_UART *)(UART12_BASE|S_BASE_OFFSET))

#define REG_UART1_NS    (*(volatile T_STM32H5XX_UART *)UART1_BASE)
#define REG_UART2_NS    (*(volatile T_STM32H5XX_UART *)UART2_BASE)
#define REG_UART3_NS    (*(volatile T_STM32H5XX_UART *)UART3_BASE)
#define REG_UART4_NS    (*(volatile T_STM32H5XX_UART *)UART4_BASE)
#define REG_UART5_NS    (*(volatile T_STM32H5XX_UART *)UART5_BASE)
#define REG_UART6_NS    (*(volatile T_STM32H5XX_UART *)UART6_BASE)
#define REG_UART7_NS    (*(volatile T_STM32H5XX_UART *)UART7_BASE)
#define REG_UART8_NS    (*(volatile T_STM32H5XX_UART *)UART8_BASE)
#define REG_UART9_NS    (*(volatile T_STM32H5XX_UART *)UART9_BASE)
#define REG_UART10_NS   (*(volatile T_STM32H5XX_UART *)UART10_BASE)
#define REG_UART11_NS   (*(volatile T_STM32H5XX_UART *)UART11_BASE)
#define REG_UART12_NS   (*(volatile T_STM32H5XX_UART *)UART12_BASE)
#else

#define REG_UART1       (*(volatile T_STM32H5XX_UART *)UART1_BASE)
#define REG_UART2       (*(volatile T_STM32H5XX_UART *)UART2_BASE)
#define REG_UART3       (*(volatile T_STM32H5XX_UART *)UART3_BASE)
#define REG_UART4       (*(volatile T_STM32H5XX_UART *)UART4_BASE)
#define REG_UART5       (*(volatile T_STM32H5XX_UART *)UART5_BASE)
#define REG_UART6       (*(volatile T_STM32H5XX_UART *)UART6_BASE)
#define REG_UART7       (*(volatile T_STM32H5XX_UART *)UART7_BASE)
#define REG_UART8       (*(volatile T_STM32H5XX_UART *)UART8_BASE)
#define REG_UART9       (*(volatile T_STM32H5XX_UART *)UART9_BASE)
#define REG_UART10      (*(volatile T_STM32H5XX_UART *)UART10_BASE)
#define REG_UART11      (*(volatile T_STM32H5XX_UART *)UART11_BASE)
#define REG_UART12      (*(volatile T_STM32H5XX_UART *)UART12_BASE)
#endif
#endif

/* UARTx_CR1 */

#define USART_CR1_RXFFIE        0x80000000
#define USART_CR1_TXFEIE        0x40000000
#define USART_CR1_FIFOE         0x20000000
#define USART_CR1_M1            0x10000000
#define USART_CR1_EOBIE         0x08000000
#define USART_CR1_RTOIE         0x04000000
#define USART_CR1_DEAT4         0x02000000
#define USART_CR1_DEAT3         0x01000000
#define USART_CR1_DEAT2         0x00800000
#define USART_CR1_DEAT1         0x00400000
#define USART_CR1_DEAT0         0x00200000
#define USART_CR1_DEAT_SHIFT    21
#define USART_CR1_DEDT4         0x00100000
#define USART_CR1_DEDT3         0x00080000
#define USART_CR1_DEDT2         0x00040000
#define USART_CR1_DEDT1         0x00020000
#define USART_CR1_DEDT0         0x00010000
#define USART_CR1_DEDT_SHIFT    16
#define USART_CR1_OVER8         0x00008000
#define USART_CR1_CMIE          0x00004000
#define USART_CR1_MME           0x00002000
#define USART_CR1_M0            0x00001000
#define USART_CR1_WAKE          0x00000800
#define USART_CR1_PCE           0x00000400
#define USART_CR1_PS            0x00000200
#define USART_CR1_PEIE          0x00000100
#define USART_CR1_TXFNFIE       0x00000080
#define USART_CR1_TCIE          0x00000040
#define USART_CR1_RXFNEIE       0x00000020
#define USART_CR1_IDLEIE        0x00000010
#define USART_CR1_TE            0x00000008
#define USART_CR1_RE            0x00000004
#define USART_CR1_UESM          
#define USART_CR1_UE            0x00000001

/* UARTx_CR2 */

#define USART_CR2_ADD_SHIFT     24
#define USART_CR2_RTOEN         0x00800000
#define USART_CR2_ABRMOD_SHIFT  21
#define USART_CR2_ABREN         0x00100000
#define USART_CR2_MSBFIRST      0x00080000
#define USART_CR2_DATAINV       0x00040000
#define USART_CR2_TXINV         0x00020000
#define USART_CR2_RXINV         0x00010000
#define USART_CR2_SWAP          0x00008000
#define USART_CR2_LINEN         0x00004000
#define USART_CR2_STOP1         (0x00 << 12)
#define USART_CR2_STOP0_5       (0x01 << 12)
#define USART_CR2_STOP2         (0x02 << 12)
#define USART_CR2_STOP1_5       (0x03 << 12)
#define USART_CR2_CLKEN         0x00000800
#define USART_CR2_CPOL          0x00000400
#define USART_CR2_CPHA          0x00000200
#define USART_CR2_LBCL          0x00000100
#define USART_CR2_LBDIE         0x00000040
#define USART_CR2_LBDL          0x00000020
#define USART_CR2_ADDM7         0x00000010
#define USART_CR2_DIS_NSS       0x00000008
#define USART_CR2_SLVEN         0x00000001

/* UARTx_CR3 */

#define USART_CR3_TXFTCFG_SHIFT 29
#define USART_CR3_RXFTIE        0x10000000
#define USART_CR3_RXFTCFG_SHIFT 25
#define USART_CR3_TCBGTIE       0x01000000
#define USART_CR3_TXFTIE        0x00800000
#define USART_CR3_WUFIE         0x00400000
#define USART_CR3_WUS1          0x00200000
#define USART_CR3_WUS0          0x00100000
#define USART_CR3_SCARCNT_SHIFT 17
#define USART_CR3_DEP           0x00008000
#define USART_CR3_DEM           0x00004000
#define USART_CR3_DDRE          0x00002000
#define USART_CR3_OVRDIS        0x00001000
#define USART_CR3_ONEBI         0x00000800
#define USART_CR3_CTSIE         0x00000400
#define USART_CR3_CTSE          0x00000200
#define USART_CR3_RTSE          0x00000100
#define USART_CR3_DMAT          0x00000080
#define USART_CR3_DMAR          0x00000040
#define USART_CR3_SCEN          0x00000020
#define USART_CR3_NACK          0x00000010
#define USART_CR3_HDSEL         0x00000008
#define USART_CR3_IRLP          0x00000004
#define USART_CR3_IREN          0x00000002
#define USART_CR3_EIE           0x00000001

/* UART_GTPR */

#define USART_GTPR_GT_SHIFT     8
#define USART_GTPR_PSC_SHIFT    0

/* UART_RTOR */

#define USART_GTPR_BLEN_SHIFT   24
#define USART_GTPR_RTO_SHIFT    0

/* UART_RQR */

#define USART_RQR_TXFRQ         0x00000010
#define USART_RQR_RXFRQ         0x00000008
#define USART_RQR_MMRQ          0x00000004
#define USART_RQR_SBKRQ         0x00000002
#define USART_RQR_ABRRQ         0x00000001

/* UART_ISR */

#define USART_ISR_TDNULL        0x80000000    /* Virtual status */
#define USART_ISR_TXFT          0x08000000
#define USART_ISR_RXFT          0x04000000
#define USART_ISR_TCBGT         0x02000000
#define USART_ISR_RXFE          0x01000000
#define USART_ISR_TXFE          0x00800000
#define USART_ISR_REACK         0x00400000
#define USART_ISR_TEACK         0x00200000
#define USART_ISR_WUF           0x00100000
#define USART_ISR_RWU           0x00080000
#define USART_ISR_SBKF          0x00040000
#define USART_ISR_CMF           0x00020000
#define USART_ISR_BUSY          0x00010000
#define USART_ISR_ABRF          0x00008000
#define USART_ISR_ABRE          0x00004000
#define USART_ISR_UDR           0x00002000
#define USART_ISR_EOBF          0x00001000
#define USART_ISR_RTOF          0x00000800
#define USART_ISR_CTS           0x00000400
#define USART_ISR_CTSIF         0x00000200
#define USART_ISR_LBDF          0x00000100
#define USART_ISR_TXE           0x00000080
#define USART_ISR_TXFNF         0x00000080
#define USART_ISR_TC            0x00000040
#define USART_ISR_RXNE          0x00000020
#define USART_ISR_RXFNE         0x00000020
#define USART_ISR_IDLE          0x00000010
#define USART_ISR_ORE           0x00000008
#define USART_ISR_NF            0x00000004
#define USART_ISR_FE            0x00000002
#define USART_ISR_PE            0x00000001

/* UART_ICR */

#define USART_ICR_WUCF          0x00100000
#define USART_ICR_CMCF          0x00020000
#define USART_ICR_UDRCF         0x00002000
#define USART_ICR_EOBCF         0x00001000
#define USART_ICR_RTOCF         0x00000800
#define USART_ICR_CTSCF         0x00000200
#define USART_ICR_LBDCF         0x00000100
#define USART_ICR_TCBGTCF       0x00000080
#define USART_ICR_TCCF          0x00000040
#define USART_ICR_TXFECF        0x00000020
#define USART_ICR_IDLECF        0x00000010
#define USART_ICR_ORECF         0x00000008
#define USART_ICR_NECF          0x00000004
#define USART_ICR_FECF          0x00000002
#define USART_ICR_PECF          0x00000001


#ifdef __cplusplus
}
#endif
#endif /* _DV_STM32H5XX_UART_H_ */
