/****************************************************************************
    MULti Core Operating System Tiny (MULCOS-T)

            Reset handler for Cortex-M33
                Used STM32CubeIDE

    Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.

    File name   start.s

    Jan/01/2024 Created.

****************************************************************************/


        .syntax unified
        .thumb

        .section    .text.Reset_Handler

        .extern     _init_clock_device
        .extern     __vector_table

/*
 *      Definitions of system register
 */

    .equ    SYSR_CCR,           0xE000ED14

    .equ    ICACHE_CR,          0x40030400
    .equ    ICACHE_SR,          0x40030404
    .equ    ICACHE_CR_ENbit,    0x00000001
    .equ    ICACHE_CR_CACHEINV, 0x00000002
    .equ    ICACHE_CR_WAYSEL,   0x00000004
    .equ    ICACHE_SR_BSYENDF,  0x00000002
    .equ    ICACHE_SR_BUSYF,    0x00000001

    .equ    DCACHE_CR,          0x40031400
    .equ    DCACHE_SR,          0x40031404
    .equ    DCACHE_CR_ENbit,    0x00000001
    .equ    DCACHE_CR_CACHEINV, 0x00000002

    .equ    FNC_RETURN,         0xFEFFFFFF

/*
 *      Power on reset
 */

        .globl      Reset_Handler
        .type       Reset_Handler, %function
Reset_Handler:
        ldr     r0,  =__main_sp
        ldr     r1,  =__main_lim_sp
        msr     msp,  r0
        msr     msplim,  r1

        /* Invalidate instruction cache */

        ldr     r1,  =ICACHE_CR
        ldr     r0,  [r1]
        orr     r0,  r0,  #ICACHE_CR_ENbit
        str     r0,  [r1]
        dsb
        isb
        ldr     r1,  =DCACHE_CR
        ldr     r0,  [r1]
        orr     r0,  r0,  #DCACHE_CR_ENbit
        str     r0,  [r1]
        dsb
        isb

/* Initialize bss and data */

        mov     r0,  #0
        /* clear bss */
        ldr     r1,  =__bss_start__
        ldr     r2,  =__bss_end__
1:
        cmp     r1,  r2
        it      lo
        strlo   r0,  [r1],  #4
        blo     1b
        /* copy .data of private */
        ldr     r0,  =_sidata
        ldr     r1,  =_sdata
        ldr     r2,  =_edata
        cmp     r1,  r2
        beq     3f
2:
        cmp     r1,  r2
        itt     lt
        ldrlt   r3,  [r0],  #4
        strlt   r3,  [r1],  #4
        blt     2b
3:

        /* Enable instruction cache */

        ldr     r1,  =SYSR_CCR
        ldr     r0,  [r1]
        orr     r0,  r0, #0x00020000    /* Set CCR.IC field */
        str     r0,  [r1]
        dsb
        isb

/* Initialize peripheral device 
        ldr     r0, =_init_clock_device
        blx     r0
 */

/* Enter the iar code */
        push    {r4,lr}
        bl      main
        pop     {r4,lr}
        ldr     r1,  =FNC_RETURN
        cmp     r1,  lr
        bne     LoopForever
        bx      lr

LoopForever:
        b       LoopForever

