/*
 *  MULti Core Operating System - Multilayer Kernel (MULCOS-X)
 *          Internal functions, only available in the secure world
 *
 *          Copyright (c) 2024 SPIRAL-TECH Co.,ltd All Rights Reserved.
 *
 *  File name   sys_func.c
 *
 *  Apr/02/2024 Created.
 *
 */

#include "kernel.h"
#include "arch_device.h"


extern UW __vector_table[];
extern UW __vector_table_rtos[];
extern void default_handler(void);


/*******************************************
 *      Defnitions of shared function
 *******************************************/

/*
 *      Initialize NVEC
 */

void initial_nvec(void)
{
    volatile uint32_t *vector;
    UINT inttype;
    UINT i, j;

    inttype = (NVIC_ICTR & 0xF) + 1;
    for (i = 0; i < inttype; i++) {
        NVIC_ICER[i] = 0xFFFFFFFF;      /* Clear enable and pending interrupt */
        NVIC_ICPR[i] = 0xFFFFFFFF;
    }
    for (i = 0; i < (inttype * 32); i++) {
        NVIC_IPR[i] = 0xFF;             /* Set lowest priority for interrupt */
    }

    vector = __vector_table_rtos;
    j = (inttype * 32) + 16;
    for(i = 0; i < 15; i++) {
        vector[i] = __vector_table[i];
    }
    for(; i < j; i++) {
        vector[i] = (uint32_t)&default_handler;
    }
    SYSR_AIRCR = 0x05FA0000 + 0x0300;   /* 4bit pre-emption priority */
    SYSR_VTOR = vector;                 /* Set vector table address */
    for (i = 0; i < 12; i++) {          /* Initialize system exception priority */
        SYSR_SHPR[i] = 0x00;
    }
}


/*
 *      Initialize Floating Point Unit
 */

void initial_fpu(void)
{
    /* Enable fault exceptions */
    SYSR_SHCSR |= ((1 << 19) | (1 << 18) | (1 << 17) | (1 << 16));

    /* Enable full access to FPU */
    SYSR_CPACR |= ((3UL << 10*2)|(3UL << 11*2));

    /* Enable automatic set FPU, and lazy saving */
    VFP_FPCCR |= FPCCR_ASPEN;
}


/*
 *      Wait EventFlag
 */

//#if ((__ARM_ARCH == 8) && defined(__BUILD_TRZ__) && !defined(__BUILD_SECURE__))
///* ARMv8-M NonSecure apllications with TrustZone */
//ER twai_flg(ID flgid, FLGPTN waiptn, MODE wfmode, FLGPTN *p_flgptn, TMO tmout)
//{
//    T_PFLG pflg;
//    ER ercd;
//
//    pflg.waiptn = waiptn;
//    pflg.wfmode = wfmode;
//    pflg.flgptn = p_flgptn;
//    pflg.tmout  = tmout;
//    ercd = vwai_flg(flgid, &pflg);
//    return ercd;
//}
//#endif


